@@ -627,7 +627,7 @@ clutter_backend_init (ClutterBackend *self)
   self->units_per_em = -1.0;
   self->units_serial = 1;
 
-  self->dummy_onscreen = COGL_INVALID_HANDLE;
+  self->dummy_onscreen = NULL;
 }
 
 void
@@ -1009,27 +1009,6 @@ _clutter_backend_get_keymap_direction (ClutterBackend *backend)
   return PANGO_DIRECTION_NEUTRAL;
 }
 
-void
-_clutter_backend_reset_cogl_framebuffer (ClutterBackend *backend)
-{
-  if (backend->dummy_onscreen == COGL_INVALID_HANDLE)
-    {
-      GError *internal_error = NULL;
-
-      backend->dummy_onscreen = cogl_onscreen_new (backend->cogl_context, 1, 1);
-
-      if (!cogl_framebuffer_allocate (COGL_FRAMEBUFFER (backend->dummy_onscreen),
-                                      &internal_error))
-        {
-          g_critical ("Unable to create dummy onscreen: %s", internal_error->message);
-          g_error_free (internal_error);
-          return;
-        }
-    }
-
-  cogl_set_framebuffer (COGL_FRAMEBUFFER (backend->dummy_onscreen));
-}
-
 void
 clutter_set_allowed_drivers (const char *drivers)
 {
