@@ -177,18 +177,18 @@ clutter_shader_effect_clear (ClutterShaderEffect *self,
 {
   ClutterShaderEffectPrivate *priv = self->priv;
 
-  if (priv->shader != COGL_INVALID_HANDLE)
+  if (priv->shader != NULL)
     {
-      cogl_handle_unref (priv->shader);
+      cogl_object_unref (priv->shader);
 
-      priv->shader = COGL_INVALID_HANDLE;
+      priv->shader = NULL;
     }
 
-  if (priv->program != COGL_INVALID_HANDLE)
+  if (priv->program != NULL)
     {
-      cogl_handle_unref (priv->program);
+      cogl_object_unref (priv->program);
 
-      priv->program = COGL_INVALID_HANDLE;
+      priv->program = NULL;
     }
 
   if (reset_uniforms && priv->uniforms != NULL)
@@ -208,7 +208,7 @@ clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
   gpointer key, value;
   gsize size;
 
-  if (priv->program == COGL_INVALID_HANDLE)
+  if (priv->program == NULL)
     return;
 
   if (priv->uniforms == NULL)
@@ -333,7 +333,7 @@ clutter_shader_effect_create_shader (ClutterShaderEffect *self)
 
     default:
       g_assert_not_reached ();
-      return COGL_INVALID_HANDLE;
+      return NULL;
     }
 }
 
@@ -353,7 +353,7 @@ clutter_shader_effect_try_static_source (ClutterShaderEffect *self)
                                   CLUTTER_TYPE_SHADER_EFFECT,
                                   ClutterShaderEffectClassPrivate);
 
-      if (class_priv->shader == COGL_INVALID_HANDLE)
+      if (class_priv->shader == NULL)
         {
           gchar *source;
 
@@ -387,15 +387,16 @@ clutter_shader_effect_try_static_source (ClutterShaderEffect *self)
             }
         }
 
-      priv->shader = cogl_handle_ref (class_priv->shader);
+      priv->shader = cogl_object_ref (class_priv->shader);
 
-      if (class_priv->program != COGL_INVALID_HANDLE)
-        priv->program = cogl_handle_ref (class_priv->program);
+      if (class_priv->program != NULL)
+        priv->program = cogl_object_ref (class_priv->program);
     }
 }
 
 static void
-clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
+clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect,
+                                    ClutterPaintContext    *paint_context)
 {
   ClutterShaderEffect *self = CLUTTER_SHADER_EFFECT (effect);
   ClutterShaderEffectPrivate *priv = self->priv;
@@ -404,13 +405,13 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
 
   /* If the source hasn't been set then we'll try to get it from the
      static source instead */
-  if (priv->shader == COGL_INVALID_HANDLE)
+  if (priv->shader == NULL)
     clutter_shader_effect_try_static_source (self);
 
   /* we haven't been prepared or we don't have support for
    * GLSL shaders in Clutter
    */
-  if (priv->program == COGL_INVALID_HANDLE)
+  if (priv->program == NULL)
     goto out;
 
   CLUTTER_NOTE (SHADER, "Applying the shader effect of type '%s'",
@@ -425,7 +426,7 @@ clutter_shader_effect_paint_target (ClutterOffscreenEffect *effect)
 out:
   /* paint the offscreen buffer */
   parent = CLUTTER_OFFSCREEN_EFFECT_CLASS (clutter_shader_effect_parent_class);
-  parent->paint_target (effect);
+  parent->paint_target (effect, paint_context);
 
 }
 
@@ -534,7 +535,7 @@ clutter_shader_effect_new (ClutterShaderType shader_type)
  * Retrieves a pointer to the shader's handle
  *
  * Return value: (transfer none): a pointer to the shader's handle,
- *   or %COGL_INVALID_HANDLE
+ *   or %NULL
  *
  * Since: 1.4
  */
@@ -542,7 +543,7 @@ CoglHandle
 clutter_shader_effect_get_shader (ClutterShaderEffect *effect)
 {
   g_return_val_if_fail (CLUTTER_IS_SHADER_EFFECT (effect),
-                        COGL_INVALID_HANDLE);
+                        NULL);
 
   return effect->priv->shader;
 }
@@ -554,7 +555,7 @@ clutter_shader_effect_get_shader (ClutterShaderEffect *effect)
  * Retrieves a pointer to the program's handle
  *
  * Return value: (transfer none): a pointer to the program's handle,
- *   or %COGL_INVALID_HANDLE
+ *   or %NULL
  *
  * Since: 1.4
  */
@@ -562,7 +563,7 @@ CoglHandle
 clutter_shader_effect_get_program (ClutterShaderEffect *effect)
 {
   g_return_val_if_fail (CLUTTER_IS_SHADER_EFFECT (effect),
-                        COGL_INVALID_HANDLE);
+                        NULL);
 
   return effect->priv->program;
 }
@@ -892,7 +893,7 @@ clutter_shader_effect_set_shader_source (ClutterShaderEffect *effect,
 
   priv = effect->priv;
 
-  if (priv->shader != COGL_INVALID_HANDLE)
+  if (priv->shader != NULL)
     return TRUE;
 
   priv->shader = clutter_shader_effect_create_shader (effect);
