@@ -202,20 +202,6 @@ cogl_renderer_set_winsys_id (CoglRenderer *renderer,
 CoglWinsysID
 cogl_renderer_get_winsys_id (CoglRenderer *renderer);
 
-/**
- * cogl_renderer_get_n_fragment_texture_units:
- * @renderer: A #CoglRenderer
- *
- * Queries how many texture units can be used from fragment programs
- *
- * Returns: the number of texture image units.
- *
- * Since: 1.8
- * Stability: Unstable
- */
-int
-cogl_renderer_get_n_fragment_texture_units (CoglRenderer *renderer);
-
 /**
  * cogl_renderer_check_onscreen_template: (skip)
  * @renderer: A #CoglRenderer
@@ -261,10 +247,6 @@ cogl_renderer_connect (CoglRenderer *renderer, GError **error);
  * @COGL_RENDERER_CONSTRAINT_USES_XLIB: Require the renderer to be X11
  *                                      based and use Xlib
  * @COGL_RENDERER_CONSTRAINT_USES_EGL: Require the renderer to be EGL based
- * @COGL_RENDERER_CONSTRAINT_SUPPORTS_COGL_GLES2: Require that the
- *    renderer supports creating a #CoglGLES2Context via
- *    cogl_gles2_context_new(). This can be used to integrate GLES 2.0
- *    code into Cogl based applications.
  *
  * These constraint flags are hard-coded features of the different renderer
  * backends. Sometimes a platform may support multiple rendering options which
@@ -286,7 +268,6 @@ typedef enum
   COGL_RENDERER_CONSTRAINT_USES_X11 = (1 << 0),
   COGL_RENDERER_CONSTRAINT_USES_XLIB = (1 << 1),
   COGL_RENDERER_CONSTRAINT_USES_EGL = (1 << 2),
-  COGL_RENDERER_CONSTRAINT_SUPPORTS_COGL_GLES2 = (1 << 3)
 } CoglRendererConstraint;
 
 
