@@ -43,7 +43,6 @@
 static void _cogl_program_free (CoglProgram *program);
 
 COGL_HANDLE_DEFINE (Program, program);
-COGL_OBJECT_DEFINE_DEPRECATED_REF_COUNTING (program);
 
 /* A CoglProgram is effectively just a list of shaders that will be
    used together and a set of values for the custom uniforms. No
@@ -60,7 +59,7 @@ _cogl_program_free (CoglProgram *program)
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* Unref all of the attached shaders and destroy the list */
-  g_slist_free_full (program->attached_shaders, cogl_handle_unref);
+  g_slist_free_full (program->attached_shaders, cogl_object_unref);
 
   for (i = 0; i < program->custom_uniforms->len; i++)
     {
@@ -97,7 +96,6 @@ cogl_program_attach_shader (CoglHandle program_handle,
                             CoglHandle shader_handle)
 {
   CoglProgram *program;
-  CoglShader *shader;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -105,15 +103,10 @@ cogl_program_attach_shader (CoglHandle program_handle,
     return;
 
   program = program_handle;
-  shader = shader_handle;
-
-  if (shader->language == COGL_SHADER_LANGUAGE_GLSL)
-    g_return_if_fail (_cogl_program_get_language (program) ==
-                      COGL_SHADER_LANGUAGE_GLSL);
 
   program->attached_shaders
     = g_slist_prepend (program->attached_shaders,
-                       cogl_handle_ref (shader_handle));
+                       cogl_object_ref (shader_handle));
 
   program->age++;
 }
@@ -131,18 +124,12 @@ cogl_program_use (CoglHandle handle)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_return_if_fail (handle == COGL_INVALID_HANDLE ||
-                    cogl_is_program (handle));
+  g_return_if_fail (handle == NULL || cogl_is_program (handle));
 
-  if (ctx->current_program == 0 && handle != 0)
-    ctx->legacy_state_set++;
-  else if (handle == 0 && ctx->current_program != 0)
-    ctx->legacy_state_set--;
-
-  if (handle != COGL_INVALID_HANDLE)
-    cogl_handle_ref (handle);
-  if (ctx->current_program != COGL_INVALID_HANDLE)
-    cogl_handle_unref (ctx->current_program);
+  if (handle != NULL)
+    cogl_object_ref (handle);
+  if (ctx->current_program != NULL)
+    cogl_object_unref (ctx->current_program);
   ctx->current_program = handle;
 }
 
@@ -357,26 +344,19 @@ _cogl_program_flush_uniforms (CoglProgram *program,
         {
           if (gl_program_changed || !uniform->location_valid)
             {
-              if (_cogl_program_get_language (program) ==
-                  COGL_SHADER_LANGUAGE_GLSL)
-                uniform->location =
-                  ctx->glGetUniformLocation (gl_program, uniform->name);
+               uniform->location =
+                 ctx->glGetUniformLocation (gl_program, uniform->name);
 
-              uniform->location_valid = TRUE;
+               uniform->location_valid = TRUE;
             }
 
           /* If the uniform isn't really in the program then there's
              no need to actually set it */
           if (uniform->location != -1)
             {
-              switch (_cogl_program_get_language (program))
-                {
-                case COGL_SHADER_LANGUAGE_GLSL:
-                  _cogl_boxed_value_set_uniform (ctx,
-                                                 uniform->location,
-                                                 &uniform->value);
-                  break;
-                }
+               _cogl_boxed_value_set_uniform (ctx,
+                                              uniform->location,
+                                              &uniform->value);
             }
 
           uniform->dirty = FALSE;
@@ -384,22 +364,6 @@ _cogl_program_flush_uniforms (CoglProgram *program,
     }
 }
 
-CoglShaderLanguage
-_cogl_program_get_language (CoglHandle handle)
-{
-  CoglProgram *program = handle;
-
-  /* Use the language of the first shader */
-
-  if (program->attached_shaders)
-    {
-      CoglShader *shader = program->attached_shaders->data;
-      return shader->language;
-    }
-  else
-    return COGL_SHADER_LANGUAGE_GLSL;
-}
-
 static gboolean
 _cogl_program_has_shader_type (CoglProgram *program,
                                CoglShaderType type)
