@@ -44,8 +44,6 @@
 #include "cogl-list.h"
 #include "driver/gl/cogl-util-gl-private.h"
 
-#ifdef COGL_PIPELINE_FRAGEND_GLSL
-
 #include "cogl-context-private.h"
 #include "cogl-object-private.h"
 #include "cogl-pipeline-cache.h"
@@ -1119,12 +1117,7 @@ const CoglPipelineFragend _cogl_pipeline_glsl_fragend =
 {
   _cogl_pipeline_fragend_glsl_start,
   _cogl_pipeline_fragend_glsl_add_layer,
-  NULL, /* passthrough */
   _cogl_pipeline_fragend_glsl_end,
   _cogl_pipeline_fragend_glsl_pre_change_notify,
-  NULL, /* pipeline_set_parent_notify */
   _cogl_pipeline_fragend_glsl_layer_pre_change_notify
 };
-
-#endif /* COGL_PIPELINE_FRAGEND_GLSL */
-
