@@ -42,8 +42,6 @@
 #include "driver/gl/cogl-util-gl-private.h"
 #include "driver/gl/cogl-pipeline-opengl-private.h"
 
-#ifdef COGL_PIPELINE_PROGEND_GLSL
-
 #include "cogl-context-private.h"
 #include "cogl-object-private.h"
 #include "cogl-pipeline-cache.h"
@@ -640,15 +638,6 @@ _cogl_pipeline_progend_glsl_flush_uniforms (CoglPipeline *pipeline,
 static gboolean
 _cogl_pipeline_progend_glsl_start (CoglPipeline *pipeline)
 {
-  CoglHandle user_program;
-
-  _COGL_GET_CONTEXT (ctx, FALSE);
-
-  user_program = cogl_pipeline_get_user_program (pipeline);
-  if (user_program &&
-      _cogl_program_get_language (user_program) != COGL_SHADER_LANGUAGE_GLSL)
-    return FALSE;
-
   return TRUE;
 }
 
@@ -744,8 +733,6 @@ _cogl_pipeline_progend_glsl_end (CoglPipeline *pipeline,
 
               _cogl_shader_compile_real (shader, pipeline);
 
-              g_assert (shader->language == COGL_SHADER_LANGUAGE_GLSL);
-
               GE( ctx, glAttachShader (program_state->program,
                                        shader->gl_handle) );
             }
@@ -773,8 +760,7 @@ _cogl_pipeline_progend_glsl_end (CoglPipeline *pipeline,
 
   gl_program = program_state->program;
 
-  _cogl_use_fragment_program (gl_program, COGL_PIPELINE_PROGRAM_TYPE_GLSL);
-  _cogl_use_vertex_program (gl_program, COGL_PIPELINE_PROGRAM_TYPE_GLSL);
+  cogl_use_program (gl_program);
 
   state.unit = 0;
   state.gl_program = gl_program;
@@ -1057,13 +1043,9 @@ update_float_uniform (CoglPipeline *pipeline,
 
 const CoglPipelineProgend _cogl_pipeline_glsl_progend =
   {
-    COGL_PIPELINE_VERTEND_GLSL,
-    COGL_PIPELINE_FRAGEND_GLSL,
     _cogl_pipeline_progend_glsl_start,
     _cogl_pipeline_progend_glsl_end,
     _cogl_pipeline_progend_glsl_pre_change_notify,
     _cogl_pipeline_progend_glsl_layer_pre_change_notify,
     _cogl_pipeline_progend_glsl_pre_paint
   };
-
-#endif /* COGL_PIPELINE_PROGEND_GLSL */
