@@ -144,7 +144,7 @@ struct _MetaBackendPrivate
   gboolean is_pointer_position_initialized;
 
   guint device_update_idle_id;
-  guint keymap_state_changed_id;
+  gulong keymap_state_changed_id;
 
   GHashTable *device_monitors;
 
@@ -184,7 +184,7 @@ meta_backend_finalize (GObject *object)
       ClutterKeymap *keymap;
 
       keymap = clutter_backend_get_keymap (priv->clutter_backend);
-      g_signal_handler_disconnect (keymap, priv->keymap_state_changed_id);
+      g_clear_signal_handler (&priv->keymap_state_changed_id, keymap);
     }
 
   g_list_free_full (priv->gpus, g_object_unref);
@@ -208,8 +208,7 @@ meta_backend_finalize (GObject *object)
   g_clear_object (&priv->system_bus);
   g_clear_object (&priv->upower_proxy);
 
-  if (priv->device_update_idle_id)
-    g_source_remove (priv->device_update_idle_id);
+  g_clear_handle_id (&priv->device_update_idle_id, g_source_remove);
 
   g_hash_table_destroy (priv->device_monitors);
 
@@ -258,7 +257,7 @@ meta_backend_monitors_changed (MetaBackend *backend)
     meta_backend_get_monitor_manager (backend);
   ClutterDeviceManager *manager = clutter_device_manager_get_default ();
   ClutterInputDevice *device = clutter_device_manager_get_core_device (manager, CLUTTER_POINTER_DEVICE);
-  ClutterPoint point;
+  graphene_point_t point;
 
   meta_backend_sync_screen_size (backend);
 
