@@ -136,7 +136,6 @@ static void
 meta_cursor_tracker_init (MetaCursorTracker *self)
 {
   self->is_showing = TRUE;
-  self->keep_focus_while_hidden = FALSE;
 }
 
 static void
@@ -408,7 +407,7 @@ get_pointer_position_clutter (int         *x,
 {
   ClutterDeviceManager *cmanager;
   ClutterInputDevice *cdevice;
-  ClutterPoint point;
+  graphene_point_t point;
 
   cmanager = clutter_device_manager_get_default ();
   cdevice = clutter_device_manager_get_core_device (cmanager, CLUTTER_POINTER_DEVICE);
@@ -458,44 +457,6 @@ meta_cursor_tracker_set_pointer_visible (MetaCursorTracker *tracker,
   g_signal_emit (tracker, signals[VISIBILITY_CHANGED], 0);
 }
 
-/**
- * meta_cursor_tracker_get_keep_focus_while_hidden:
- * @tracker: a #MetaCursorTracker object.
- *
- * Returns: %FALSE if the Wayland focus surface of the pointer will
- * be forced to NULL while the pointer is hidden, %TRUE otherwise.
- * This function is only meant to be used by the magnifier of the shell
- * and will be removed in a future release.
- */
-gboolean
-meta_cursor_tracker_get_keep_focus_while_hidden (MetaCursorTracker *tracker)
-{
-  return tracker->keep_focus_while_hidden;
-}
-
-/**
- * meta_cursor_tracker_set_keep_focus_while_hidden:
- * @tracker: a #MetaCursorTracker object.
- * @keep_focus: whether to keep the cursor focus while hidden
- *
- * If this is set to %TRUE, the Wayland focus surface of the pointer will
- * not be forced to NULL while the pointer is hidden.
- * This function is only meant to be used by the magnifier of the shell
- * and will be removed in a future release.
- */
-void
-meta_cursor_tracker_set_keep_focus_while_hidden (MetaCursorTracker *tracker,
-                                                 gboolean           keep_focus)
-{
-  if (keep_focus == tracker->keep_focus_while_hidden)
-    return;
-  tracker->keep_focus_while_hidden = keep_focus;
-
-  sync_cursor (tracker);
-
-  g_signal_emit (tracker, signals[VISIBILITY_CHANGED], 0);
-}
-
 MetaCursorSprite *
 meta_cursor_tracker_get_displayed_cursor (MetaCursorTracker *tracker)
 {
