@@ -101,8 +101,9 @@ MetaMonitorTiled * meta_monitor_tiled_new (MetaGpu            *gpu,
                                            MetaOutput         *output);
 
 META_EXPORT_TEST
-MetaMonitorNormal * meta_monitor_normal_new (MetaGpu    *gpu,
-                                             MetaOutput *output);
+MetaMonitorNormal * meta_monitor_normal_new (MetaGpu            *gpu,
+                                             MetaMonitorManager *monitor_manager,
+                                             MetaOutput         *output);
 
 META_EXPORT_TEST
 MetaMonitorSpec * meta_monitor_get_spec (MetaMonitor *monitor);
@@ -280,4 +281,6 @@ int meta_monitor_spec_compare (MetaMonitorSpec *monitor_spec_a,
 META_EXPORT_TEST
 void meta_monitor_spec_free (MetaMonitorSpec *monitor_id);
 
+const char * meta_monitor_get_display_name (MetaMonitor *monitor);
+
 #endif /* META_MONITOR_H */
