@@ -48,9 +48,9 @@ struct _MetaScreenCastWindowInterface
 
   gboolean (*transform_cursor_position) (MetaScreenCastWindow *screen_cast_window,
                                          MetaCursorSprite     *cursor_sprite,
-                                         ClutterPoint         *cursor_position,
+                                         graphene_point_t     *cursor_position,
                                          float                *out_cursor_scale,
-                                         ClutterPoint         *out_relative_cursor_position);
+                                         graphene_point_t     *out_relative_cursor_position);
 
   void (*capture_into) (MetaScreenCastWindow *screen_cast_window,
                         MetaRectangle        *bounds,
@@ -70,9 +70,9 @@ void meta_screen_cast_window_transform_relative_position (MetaScreenCastWindow *
 
 gboolean meta_screen_cast_window_transform_cursor_position (MetaScreenCastWindow *screen_cast_window,
                                                             MetaCursorSprite     *cursor_sprite,
-                                                            ClutterPoint         *cursor_position,
+                                                            graphene_point_t     *cursor_position,
                                                             float                *out_cursor_scale,
-                                                            ClutterPoint         *out_relative_cursor_position);
+                                                            graphene_point_t     *out_relative_cursor_position);
 
 void meta_screen_cast_window_capture_into (MetaScreenCastWindow *screen_cast_window,
                                            MetaRectangle        *bounds,
