@@ -72,7 +72,7 @@ struct _MetaBackendNative
   MetaKms *kms;
   MetaBarrierManagerNative *barrier_manager;
 
-  guint udev_device_added_handler_id;
+  gulong udev_device_added_handler_id;
 };
 
 static GInitableIface *initable_parent_iface;
@@ -150,7 +150,7 @@ constrain_all_screen_monitors (ClutterInputDevice *device,
                                float              *x,
                                float              *y)
 {
-  ClutterPoint current;
+  graphene_point_t current;
   float cx, cy;
   GList *logical_monitors, *l;
 
@@ -583,7 +583,6 @@ on_udev_device_added (MetaUdev          *udev,
     {
       g_warning ("Failed to hotplug secondary gpu '%s': %s",
                  device_path, error->message);
-      g_error_free (error);
       return;
     }
 
@@ -601,9 +600,7 @@ connect_udev_device_added_handler (MetaBackendNative *native)
 static void
 disconnect_udev_device_added_handler (MetaBackendNative *native)
 {
-  g_signal_handler_disconnect (native->udev,
-                               native->udev_device_added_handler_id);
-  native->udev_device_added_handler_id = 0;
+  g_clear_signal_handler (&native->udev_device_added_handler_id, native->udev);
 }
 
 static gboolean
