@@ -116,7 +116,8 @@ set_clip_region (MetaSurfaceActor *surface_actor,
 }
 
 static void
-meta_surface_actor_paint (ClutterActor *actor)
+meta_surface_actor_paint (ClutterActor        *actor,
+                          ClutterPaintContext *paint_context)
 {
   MetaSurfaceActor *surface_actor = META_SURFACE_ACTOR (actor);
   MetaSurfaceActorPrivate *priv =
@@ -125,12 +126,13 @@ meta_surface_actor_paint (ClutterActor *actor)
   if (priv->clip_region && cairo_region_is_empty (priv->clip_region))
     return;
 
-  CLUTTER_ACTOR_CLASS (meta_surface_actor_parent_class)->paint (actor);
+  CLUTTER_ACTOR_CLASS (meta_surface_actor_parent_class)->paint (actor,
+                                                                paint_context);
 }
 
 static void
 meta_surface_actor_pick (ClutterActor       *actor,
-                         const ClutterColor *color)
+                         ClutterPickContext *pick_context)
 {
   MetaSurfaceActor *self = META_SURFACE_ACTOR (actor);
   MetaSurfaceActorPrivate *priv =
@@ -143,7 +145,12 @@ meta_surface_actor_pick (ClutterActor       *actor,
 
   /* If there is no region then use the regular pick */
   if (priv->input_region == NULL)
-    CLUTTER_ACTOR_CLASS (meta_surface_actor_parent_class)->pick (actor, color);
+    {
+      ClutterActorClass *actor_class =
+        CLUTTER_ACTOR_CLASS (meta_surface_actor_parent_class);
+
+      actor_class->pick (actor, pick_context);
+    }
   else
     {
       int n_rects;
@@ -162,14 +169,14 @@ meta_surface_actor_pick (ClutterActor       *actor,
           box.y1 = rect.y;
           box.x2 = rect.x + rect.width;
           box.y2 = rect.y + rect.height;
-          clutter_actor_pick_box (actor, &box);
+          clutter_actor_pick_box (actor, pick_context, &box);
         }
     }
 
   clutter_actor_iter_init (&iter, actor);
 
   while (clutter_actor_iter_next (&iter, &child))
-    clutter_actor_paint (child);
+    clutter_actor_pick (child, pick_context);
 }
 
 static gboolean
@@ -221,6 +228,12 @@ meta_surface_actor_class_init (MetaSurfaceActorClass *klass)
                                         G_TYPE_NONE, 0);
 }
 
+gboolean
+meta_surface_actor_is_opaque (MetaSurfaceActor *self)
+{
+  return META_SURFACE_ACTOR_GET_CLASS (self)->is_opaque (self);
+}
+
 static void
 meta_surface_actor_cull_out (MetaCullable   *cullable,
                              cairo_region_t *unobscured_region,
@@ -481,38 +494,6 @@ meta_surface_actor_pre_paint (MetaSurfaceActor *self)
   META_SURFACE_ACTOR_GET_CLASS (self)->pre_paint (self);
 }
 
-gboolean
-meta_surface_actor_is_argb32 (MetaSurfaceActor *self)
-{
-  MetaShapedTexture *stex = meta_surface_actor_get_texture (self);
-  CoglTexture *texture = meta_shaped_texture_get_texture (stex);
-
-  /* If we don't have a texture, like during initialization, assume
-   * that we're ARGB32.
-   *
-   * If we are unredirected and we have no texture assume that we are
-   * not ARGB32 otherwise we wouldn't be unredirected in the first
-   * place. This prevents us from continually redirecting and
-   * unredirecting on every paint.
-   */
-  if (!texture)
-    return !meta_surface_actor_is_unredirected (self);
-
-  switch (cogl_texture_get_components (texture))
-    {
-    case COGL_TEXTURE_COMPONENTS_A:
-    case COGL_TEXTURE_COMPONENTS_RGBA:
-      return TRUE;
-    case COGL_TEXTURE_COMPONENTS_RG:
-    case COGL_TEXTURE_COMPONENTS_RGB:
-    case COGL_TEXTURE_COMPONENTS_DEPTH:
-      return FALSE;
-    default:
-      g_assert_not_reached ();
-      return FALSE;
-    }
-}
-
 gboolean
 meta_surface_actor_is_visible (MetaSurfaceActor *self)
 {
@@ -546,25 +527,6 @@ meta_surface_actor_set_frozen (MetaSurfaceActor *self,
     }
 }
 
-gboolean
-meta_surface_actor_should_unredirect (MetaSurfaceActor *self)
-{
-  return META_SURFACE_ACTOR_GET_CLASS (self)->should_unredirect (self);
-}
-
-void
-meta_surface_actor_set_unredirected (MetaSurfaceActor *self,
-                                     gboolean          unredirected)
-{
-  META_SURFACE_ACTOR_GET_CLASS (self)->set_unredirected (self, unredirected);
-}
-
-gboolean
-meta_surface_actor_is_unredirected (MetaSurfaceActor *self)
-{
-  return META_SURFACE_ACTOR_GET_CLASS (self)->is_unredirected (self);
-}
-
 MetaWindow *
 meta_surface_actor_get_window (MetaSurfaceActor *self)
 {
@@ -582,8 +544,8 @@ meta_surface_actor_set_transform (MetaSurfaceActor     *self,
 }
 
 void
-meta_surface_actor_set_viewport_src_rect (MetaSurfaceActor  *self,
-                                          ClutterRect       *src_rect)
+meta_surface_actor_set_viewport_src_rect (MetaSurfaceActor *self,
+                                          graphene_rect_t  *src_rect)
 {
   MetaSurfaceActorPrivate *priv =
     meta_surface_actor_get_instance_private (self);
