@@ -25,6 +25,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
+#include "core/main-private.h"
 #include "meta/common.h"
 #include "meta/types.h"
 #include "ui/theme-private.h"
@@ -36,7 +37,6 @@ typedef enum
   META_FRAME_CONTROL_TITLE,
   META_FRAME_CONTROL_DELETE,
   META_FRAME_CONTROL_MENU,
-  META_FRAME_CONTROL_APPMENU,
   META_FRAME_CONTROL_MINIMIZE,
   META_FRAME_CONTROL_MAXIMIZE,
   META_FRAME_CONTROL_UNMAXIMIZE,
@@ -88,6 +88,8 @@ struct _MetaFrames
 {
   GtkWindow parent_instance;
 
+  MetaX11Display *x11_display;
+
   GHashTable *text_heights;
 
   GHashTable *frames;
@@ -112,7 +114,7 @@ struct _MetaFramesClass
 
 GType        meta_frames_get_type               (void) G_GNUC_CONST;
 
-MetaFrames *meta_frames_new (void);
+MetaFrames * meta_frames_new (MetaX11Display *x11_display);
 
 MetaUIFrame * meta_frames_manage_window (MetaFrames *frames,
                                          MetaWindow *meta_window,
