@@ -65,6 +65,7 @@ struct _MetaWaylandDataOffer
 
 typedef struct _MetaWaylandDataSourcePrivate
 {
+  struct wl_resource *resource;
   MetaWaylandDataOffer *offer;
   struct wl_array mime_types;
   gboolean has_target;
@@ -76,30 +77,21 @@ typedef struct _MetaWaylandDataSourcePrivate
   guint in_ask : 1;
 } MetaWaylandDataSourcePrivate;
 
-typedef struct _MetaWaylandDataSourceWayland
-{
-  MetaWaylandDataSource parent;
-
-  struct wl_resource *resource;
-} MetaWaylandDataSourceWayland;
-
 typedef struct _MetaWaylandDataSourcePrimary
 {
-  MetaWaylandDataSourceWayland parent;
+  MetaWaylandDataSource parent;
 } MetaWaylandDataSourcePrimary;
 
 G_DEFINE_TYPE_WITH_PRIVATE (MetaWaylandDataSource, meta_wayland_data_source,
                             G_TYPE_OBJECT);
-G_DEFINE_TYPE (MetaWaylandDataSourceWayland, meta_wayland_data_source_wayland,
-               META_TYPE_WAYLAND_DATA_SOURCE);
 G_DEFINE_TYPE (MetaWaylandDataSourcePrimary, meta_wayland_data_source_primary,
-               META_TYPE_WAYLAND_DATA_SOURCE_WAYLAND);
+               META_TYPE_WAYLAND_DATA_SOURCE);
 
 static void unset_selection_source (MetaWaylandDataDevice *data_device,
                                     MetaSelectionType      selection_type);
 
 static MetaWaylandDataSource *
-meta_wayland_data_source_wayland_new (struct wl_resource *resource);
+meta_wayland_data_source_new (struct wl_resource *resource);
 static MetaWaylandDataSource *
 meta_wayland_data_source_primary_new (struct wl_resource *resource);
 
@@ -189,6 +181,25 @@ data_offer_update_action (MetaWaylandDataOffer *offer)
     }
 }
 
+static struct wl_resource *
+meta_wayland_data_source_get_resource (MetaWaylandDataSource *source)
+{
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
+
+  return priv->resource;
+}
+
+static void
+meta_wayland_data_source_set_resource (MetaWaylandDataSource *source,
+                                       struct wl_resource    *resource)
+{
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
+
+  priv->resource = resource;
+}
+
 static void
 meta_wayland_data_source_target (MetaWaylandDataSource *source,
                                  const char *mime_type)
@@ -571,12 +582,7 @@ destroy_data_offer (struct wl_resource *resource)
               WL_DATA_OFFER_ACTION_SINCE_VERSION)
             meta_wayland_data_source_notify_finish (offer->source);
           else
-            {
-              if (seat->data_device.dnd_data_source == offer->source)
-                unset_selection_source (&seat->data_device, META_SELECTION_DND);
-              meta_wayland_data_source_cancel (offer->source);
-              meta_wayland_data_source_set_current_offer (offer->source, NULL);
-            }
+            meta_wayland_data_source_set_current_offer (offer->source, NULL);
         }
 
       g_object_remove_weak_pointer (G_OBJECT (offer->source),
@@ -659,12 +665,10 @@ data_source_set_actions (struct wl_client   *client,
   MetaWaylandDataSource *source = wl_resource_get_user_data (resource);
   MetaWaylandDataSourcePrivate *priv =
     meta_wayland_data_source_get_instance_private (source);
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
 
   if (priv->actions_set)
     {
-      wl_resource_post_error (source_wayland->resource,
+      wl_resource_post_error (priv->resource,
                               WL_DATA_SOURCE_ERROR_INVALID_ACTION_MASK,
                               "cannot set actions more than once");
       return;
@@ -672,7 +676,7 @@ data_source_set_actions (struct wl_client   *client,
 
   if (dnd_actions & ~(ALL_ACTIONS))
     {
-      wl_resource_post_error (source_wayland->resource,
+      wl_resource_post_error (priv->resource,
                               WL_DATA_SOURCE_ERROR_INVALID_ACTION_MASK,
                               "invalid actions mask %x", dnd_actions);
       return;
@@ -680,7 +684,7 @@ data_source_set_actions (struct wl_client   *client,
 
   if (meta_wayland_data_source_get_seat (source))
     {
-      wl_resource_post_error (source_wayland->resource,
+      wl_resource_post_error (priv->resource,
                               WL_DATA_SOURCE_ERROR_INVALID_ACTION_MASK,
                               "invalid action change after "
                               "wl_data_device.start_drag");
@@ -776,8 +780,8 @@ destroy_drag_focus (struct wl_listener *listener, void *data)
 
   grab->drag_focus_data_device = NULL;
 
-  g_signal_handler_disconnect (grab->drag_focus,
-                               grab->drag_focus_destroy_handler_id);
+  g_clear_signal_handler (&grab->drag_focus_destroy_handler_id,
+                          grab->drag_focus);
   grab->drag_focus = NULL;
 }
 
@@ -848,8 +852,8 @@ meta_wayland_drag_grab_set_focus (MetaWaylandDragGrab *drag_grab,
   if (drag_grab->drag_focus)
     {
       meta_wayland_surface_drag_dest_focus_out (drag_grab->drag_focus);
-      g_signal_handler_disconnect (drag_grab->drag_focus,
-                                   drag_grab->drag_focus_destroy_handler_id);
+      g_clear_signal_handler (&drag_grab->drag_focus_destroy_handler_id,
+                              drag_grab->drag_focus);
       drag_grab->drag_focus = NULL;
     }
 
@@ -1036,6 +1040,7 @@ drag_grab_button (MetaWaylandPointerGrab *grab,
       gboolean success;
 
       if (drag_grab->drag_focus && source &&
+          meta_wayland_data_source_get_current_offer (source) &&
           meta_wayland_data_source_has_target (source) &&
           meta_wayland_data_source_get_current_action (source))
         {
@@ -1059,7 +1064,6 @@ drag_grab_button (MetaWaylandPointerGrab *grab,
         }
       else
         {
-          meta_wayland_data_source_cancel (source);
           meta_wayland_data_source_set_current_offer (source, NULL);
           meta_wayland_data_device_set_dnd_source (&seat->data_device, NULL);
           unset_selection_source (&seat->data_device, META_SELECTION_DND);
@@ -1134,8 +1138,6 @@ drag_grab_data_source_destroyed (gpointer data, GObject *where_the_object_was)
   MetaWaylandDragGrab *drag_grab = data;
 
   drag_grab->drag_data_source = NULL;
-  meta_wayland_data_device_set_dnd_source (&drag_grab->seat->data_device, NULL);
-  unset_selection_source (&drag_grab->seat->data_device, META_SELECTION_DND);
   data_device_end_drag_grab (drag_grab);
 }
 
@@ -1161,7 +1163,7 @@ meta_wayland_data_device_start_drag (MetaWaylandDataDevice                 *data
 {
   MetaWaylandSeat *seat = wl_container_of (data_device, seat, data_device);
   MetaWaylandDragGrab *drag_grab;
-  ClutterPoint pos, surface_pos;
+  graphene_point_t pos, surface_pos;
   ClutterModifierType modifiers;
   MetaSurfaceActor *surface_actor;
 
@@ -1318,7 +1320,6 @@ selection_data_source_destroyed (gpointer data, GObject *object_was_here)
         wl_data_device_send_selection (data_device_resource, NULL);
     }
 
-  wl_signal_emit (&data_device->selection_ownership_signal, NULL);
   unset_selection_source (data_device, META_SELECTION_CLIPBOARD);
 }
 
@@ -1327,10 +1328,10 @@ meta_wayland_source_send (MetaWaylandDataSource *source,
                           const gchar           *mime_type,
                           gint                   fd)
 {
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
 
-  wl_data_source_send_send (source_wayland->resource, mime_type, fd);
+  wl_data_source_send_send (priv->resource, mime_type, fd);
   close (fd);
 }
 
@@ -1338,43 +1339,43 @@ static void
 meta_wayland_source_target (MetaWaylandDataSource *source,
                             const gchar           *mime_type)
 {
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
 
-  wl_data_source_send_target (source_wayland->resource, mime_type);
+  wl_data_source_send_target (priv->resource, mime_type);
 }
 
 static void
 meta_wayland_source_cancel (MetaWaylandDataSource *source)
 {
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
 
-  if (source_wayland->resource)
-    wl_data_source_send_cancelled (source_wayland->resource);
+  if (priv->resource)
+    wl_data_source_send_cancelled (priv->resource);
 }
 
 static void
 meta_wayland_source_action (MetaWaylandDataSource                  *source,
                             enum wl_data_device_manager_dnd_action  action)
 {
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
 
-  if (wl_resource_get_version (source_wayland->resource) >=
+  if (wl_resource_get_version (priv->resource) >=
       WL_DATA_SOURCE_ACTION_SINCE_VERSION)
-    wl_data_source_send_action (source_wayland->resource, action);
+    wl_data_source_send_action (priv->resource, action);
 }
 
 static void
 meta_wayland_source_drop_performed (MetaWaylandDataSource *source)
 {
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
 
-  if (wl_resource_get_version (source_wayland->resource) >=
+  if (wl_resource_get_version (priv->resource) >=
       WL_DATA_SOURCE_DND_DROP_PERFORMED_SINCE_VERSION)
-    wl_data_source_send_dnd_drop_performed (source_wayland->resource);
+    wl_data_source_send_dnd_drop_performed (priv->resource);
 }
 
 static void
@@ -1382,8 +1383,8 @@ meta_wayland_source_drag_finished (MetaWaylandDataSource *source)
 {
   MetaWaylandCompositor *compositor = meta_wayland_compositor_get_default ();
   MetaWaylandDataDevice *data_device = &compositor->seat->data_device;
-  MetaWaylandDataSourceWayland *source_wayland =
-    META_WAYLAND_DATA_SOURCE_WAYLAND (source);
+  MetaWaylandDataSourcePrivate *priv =
+    meta_wayland_data_source_get_instance_private (source);
   enum wl_data_device_manager_dnd_action action;
 
   if (meta_wayland_source_get_in_ask (source))
@@ -1392,81 +1393,15 @@ meta_wayland_source_drag_finished (MetaWaylandDataSource *source)
       meta_wayland_source_action (source, action);
     }
 
-  if (wl_resource_get_version (source_wayland->resource) >=
+  if (wl_resource_get_version (priv->resource) >=
       WL_DATA_SOURCE_DND_FINISHED_SINCE_VERSION)
-    wl_data_source_send_dnd_finished (source_wayland->resource);
+    wl_data_source_send_dnd_finished (priv->resource);
 
   unset_selection_source (data_device, META_SELECTION_DND);
 }
 
 static void
 meta_wayland_source_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (meta_wayland_data_source_wayland_parent_class)->finalize (object);
-}
-
-static void
-meta_wayland_data_source_wayland_init (MetaWaylandDataSourceWayland *source_wayland)
-{
-}
-
-static void
-meta_wayland_data_source_wayland_class_init (MetaWaylandDataSourceWaylandClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  MetaWaylandDataSourceClass *data_source_class =
-    META_WAYLAND_DATA_SOURCE_CLASS (klass);
-
-  object_class->finalize = meta_wayland_source_finalize;
-
-  data_source_class->send = meta_wayland_source_send;
-  data_source_class->target = meta_wayland_source_target;
-  data_source_class->cancel = meta_wayland_source_cancel;
-  data_source_class->action = meta_wayland_source_action;
-  data_source_class->drop_performed = meta_wayland_source_drop_performed;
-  data_source_class->drag_finished = meta_wayland_source_drag_finished;
-}
-
-static void
-meta_wayland_data_source_primary_send (MetaWaylandDataSource *source,
-                                       const gchar           *mime_type,
-                                       gint                   fd)
-{
-  MetaWaylandDataSourceWayland *source_wayland;
-
-  source_wayland = META_WAYLAND_DATA_SOURCE_WAYLAND (source);
-  gtk_primary_selection_source_send_send (source_wayland->resource,
-                                          mime_type, fd);
-  close (fd);
-}
-
-static void
-meta_wayland_data_source_primary_cancel (MetaWaylandDataSource *source)
-{
-  MetaWaylandDataSourceWayland *source_wayland;
-
-  source_wayland = META_WAYLAND_DATA_SOURCE_WAYLAND (source);
-  if (source_wayland->resource)
-    gtk_primary_selection_source_send_cancelled (source_wayland->resource);
-}
-
-static void
-meta_wayland_data_source_primary_init (MetaWaylandDataSourcePrimary *source_primary)
-{
-}
-
-static void
-meta_wayland_data_source_primary_class_init (MetaWaylandDataSourcePrimaryClass *klass)
-{
-  MetaWaylandDataSourceClass *data_source_class =
-    META_WAYLAND_DATA_SOURCE_CLASS (klass);
-
-  data_source_class->send = meta_wayland_data_source_primary_send;
-  data_source_class->cancel = meta_wayland_data_source_primary_cancel;
-}
-
-static void
-meta_wayland_data_source_finalize (GObject *object)
 {
   MetaWaylandDataSource *source = META_WAYLAND_DATA_SOURCE (object);
   MetaWaylandDataSourcePrivate *priv =
@@ -1495,7 +1430,49 @@ meta_wayland_data_source_class_init (MetaWaylandDataSourceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->finalize = meta_wayland_data_source_finalize;
+  object_class->finalize = meta_wayland_source_finalize;
+
+  klass->send = meta_wayland_source_send;
+  klass->target = meta_wayland_source_target;
+  klass->cancel = meta_wayland_source_cancel;
+  klass->action = meta_wayland_source_action;
+  klass->drop_performed = meta_wayland_source_drop_performed;
+  klass->drag_finished = meta_wayland_source_drag_finished;
+}
+
+static void
+meta_wayland_data_source_primary_send (MetaWaylandDataSource *source,
+                                       const gchar           *mime_type,
+                                       gint                   fd)
+{
+  struct wl_resource *resource = meta_wayland_data_source_get_resource (source);
+
+  gtk_primary_selection_source_send_send (resource, mime_type, fd);
+  close (fd);
+}
+
+static void
+meta_wayland_data_source_primary_cancel (MetaWaylandDataSource *source)
+{
+  struct wl_resource *resource = meta_wayland_data_source_get_resource (source);
+
+  if (resource)
+    gtk_primary_selection_source_send_cancelled (resource);
+}
+
+static void
+meta_wayland_data_source_primary_init (MetaWaylandDataSourcePrimary *source_primary)
+{
+}
+
+static void
+meta_wayland_data_source_primary_class_init (MetaWaylandDataSourcePrimaryClass *klass)
+{
+  MetaWaylandDataSourceClass *data_source_class =
+    META_WAYLAND_DATA_SOURCE_CLASS (klass);
+
+  data_source_class->send = meta_wayland_data_source_primary_send;
+  data_source_class->cancel = meta_wayland_data_source_primary_cancel;
 }
 
 static void
@@ -1598,6 +1575,16 @@ meta_wayland_data_device_get_drag_dest_funcs (void)
   return &meta_wayland_drag_dest_funcs;
 }
 
+static void
+dnd_data_source_destroyed (gpointer  data,
+                           GObject  *object_was_here)
+{
+  MetaWaylandDataDevice *data_device = data;
+
+  data_device->dnd_data_source = NULL;
+  unset_selection_source (data_device, META_SELECTION_DND);
+}
+
 void
 meta_wayland_data_device_set_dnd_source (MetaWaylandDataDevice *data_device,
                                          MetaWaylandDataSource *source)
@@ -1606,16 +1593,20 @@ meta_wayland_data_device_set_dnd_source (MetaWaylandDataDevice *data_device,
     return;
 
   if (data_device->dnd_data_source)
-    g_object_remove_weak_pointer (G_OBJECT (data_device->dnd_data_source),
-                                  (gpointer *)&data_device->dnd_data_source);
+    {
+      g_object_weak_unref (G_OBJECT (data_device->dnd_data_source),
+                           dnd_data_source_destroyed,
+                           data_device);
+    }
 
   data_device->dnd_data_source = source;
 
   if (source)
-    g_object_add_weak_pointer (G_OBJECT (data_device->dnd_data_source),
-                               (gpointer *)&data_device->dnd_data_source);
-
-  wl_signal_emit (&data_device->dnd_ownership_signal, source);
+    {
+      g_object_weak_ref (G_OBJECT (source),
+                         dnd_data_source_destroyed,
+                         data_device);
+    }
 }
 
 void
@@ -1673,8 +1664,6 @@ meta_wayland_data_device_set_selection (MetaWaylandDataDevice *data_device,
           wl_data_device_send_selection (data_device_resource, offer);
         }
     }
-
-  wl_signal_emit (&data_device->selection_ownership_signal, source);
 }
 
 static void
@@ -1684,6 +1673,7 @@ data_device_set_selection (struct wl_client *client,
                            guint32 serial)
 {
   MetaWaylandDataDevice *data_device = wl_resource_get_user_data (resource);
+  MetaWaylandSeat *seat = wl_container_of (data_device, seat, data_device);
   MetaWaylandDataSourcePrivate *priv;
   MetaWaylandDataSource *source;
 
@@ -1705,6 +1695,10 @@ data_device_set_selection (struct wl_client *client,
         }
     }
 
+  if (wl_resource_get_client (resource) !=
+      meta_wayland_keyboard_get_focus_client (seat->keyboard))
+    return;
+
   /* FIXME: Store serial and check against incoming serial here. */
   meta_wayland_data_device_set_selection (data_device, source, serial);
 }
@@ -1735,7 +1729,6 @@ primary_source_destroyed (gpointer  data,
         gtk_primary_selection_device_send_selection (data_device_resource, NULL);
     }
 
-  wl_signal_emit (&data_device->primary_ownership_signal, NULL);
   unset_selection_source (data_device, META_SELECTION_PRIMARY);
 }
 
@@ -1749,16 +1742,7 @@ meta_wayland_data_device_set_primary (MetaWaylandDataDevice *data_device,
   struct wl_client *focus_client;
   MetaSelectionSource *selection_source;
 
-  if (META_IS_WAYLAND_DATA_SOURCE_PRIMARY (source))
-    {
-      struct wl_resource *resource;
-
-      resource = META_WAYLAND_DATA_SOURCE_WAYLAND (source)->resource;
-
-      if (wl_resource_get_client (resource) !=
-          meta_wayland_keyboard_get_focus_client (seat->keyboard))
-        return;
-    }
+  g_assert (!source || META_IS_WAYLAND_DATA_SOURCE_PRIMARY (source));
 
   if (data_device->primary_data_source &&
       data_device->primary_serial - serial < UINT32_MAX / 2)
@@ -1804,8 +1788,6 @@ meta_wayland_data_device_set_primary (MetaWaylandDataDevice *data_device,
           gtk_primary_selection_device_send_selection (data_device_resource, offer);
         }
     }
-
-  wl_signal_emit (&data_device->primary_ownership_signal, source);
 }
 
 static void
@@ -1815,10 +1797,16 @@ primary_device_set_selection (struct wl_client   *client,
                               uint32_t            serial)
 {
   MetaWaylandDataDevice *data_device = wl_resource_get_user_data (resource);
+  MetaWaylandSeat *seat = wl_container_of (data_device, seat, data_device);
   MetaWaylandDataSource *source = NULL;
 
   if (source_resource)
     source = wl_resource_get_user_data (source_resource);
+
+  if (wl_resource_get_client (resource) !=
+      meta_wayland_keyboard_get_focus_client (seat->keyboard))
+    return;
+
   meta_wayland_data_device_set_primary (data_device, source, serial);
 }
 
@@ -1830,9 +1818,9 @@ static const struct gtk_primary_selection_device_interface primary_device_interf
 static void
 destroy_data_source (struct wl_resource *resource)
 {
-  MetaWaylandDataSourceWayland *source = wl_resource_get_user_data (resource);
+  MetaWaylandDataSource *source = wl_resource_get_user_data (resource);
 
-  source->resource = NULL;
+  meta_wayland_data_source_set_resource (source, NULL);
   g_object_unref (source);
 }
 
@@ -1844,7 +1832,7 @@ create_data_source (struct wl_client *client,
 
   source_resource = wl_resource_create (client, &wl_data_source_interface,
                                         wl_resource_get_version (resource), id);
-  meta_wayland_data_source_wayland_new (source_resource);
+  meta_wayland_data_source_new (source_resource);
 }
 
 static void
@@ -1936,9 +1924,9 @@ static const struct wl_data_device_manager_interface manager_interface = {
 static void
 destroy_primary_source (struct wl_resource *resource)
 {
-  MetaWaylandDataSourceWayland *source = wl_resource_get_user_data (resource);
+  MetaWaylandDataSource *source = wl_resource_get_user_data (resource);
 
-  source->resource = NULL;
+  meta_wayland_data_source_set_resource (source, NULL);
   g_object_unref (source);
 }
 
@@ -2022,9 +2010,6 @@ meta_wayland_data_device_init (MetaWaylandDataDevice *data_device)
 {
   wl_list_init (&data_device->resource_list);
   wl_list_init (&data_device->primary_resource_list);
-  wl_signal_init (&data_device->selection_ownership_signal);
-  wl_signal_init (&data_device->primary_ownership_signal);
-  wl_signal_init (&data_device->dnd_ownership_signal);
 }
 
 static struct wl_resource *
@@ -2159,29 +2144,29 @@ meta_wayland_data_source_has_mime_type (const MetaWaylandDataSource *source,
 }
 
 static MetaWaylandDataSource *
-meta_wayland_data_source_wayland_new (struct wl_resource *resource)
+meta_wayland_data_source_new (struct wl_resource *resource)
 {
-  MetaWaylandDataSourceWayland *source_wayland =
-   g_object_new (META_TYPE_WAYLAND_DATA_SOURCE_WAYLAND, NULL);
+  MetaWaylandDataSource *source =
+   g_object_new (META_TYPE_WAYLAND_DATA_SOURCE, NULL);
 
-  source_wayland->resource = resource;
+  meta_wayland_data_source_set_resource (source, resource);
   wl_resource_set_implementation (resource, &data_source_interface,
-                                  source_wayland, destroy_data_source);
+                                  source, destroy_data_source);
 
-  return META_WAYLAND_DATA_SOURCE (source_wayland);
+  return source;
 }
 
 static MetaWaylandDataSource *
 meta_wayland_data_source_primary_new (struct wl_resource *resource)
 {
-  MetaWaylandDataSourceWayland *source_primary =
+  MetaWaylandDataSource *source_primary =
     g_object_new (META_TYPE_WAYLAND_DATA_SOURCE_PRIMARY, NULL);
 
-  source_primary->resource = resource;
+  meta_wayland_data_source_set_resource (source_primary, resource);
   wl_resource_set_implementation (resource, &primary_source_interface,
                                   source_primary, destroy_primary_source);
 
-  return META_WAYLAND_DATA_SOURCE (source_primary);
+  return source_primary;
 }
 
 gboolean
