@@ -100,22 +100,13 @@ meta_wayland_shell_surface_set_window (MetaWaylandShellSurface *shell_surface,
 }
 
 void
-meta_wayland_shell_surface_configure (MetaWaylandShellSurface *shell_surface,
-                                      int                      new_x,
-                                      int                      new_y,
-                                      int                      new_width,
-                                      int                      new_height,
-                                      MetaWaylandSerial       *sent_serial)
+meta_wayland_shell_surface_configure (MetaWaylandShellSurface        *shell_surface,
+                                      MetaWaylandWindowConfiguration *configuration)
 {
   MetaWaylandShellSurfaceClass *shell_surface_class =
     META_WAYLAND_SHELL_SURFACE_GET_CLASS (shell_surface);
 
-  shell_surface_class->configure (shell_surface,
-                                  new_x,
-                                  new_y,
-                                  new_width,
-                                  new_height,
-                                  sent_serial);
+  shell_surface_class->configure (shell_surface, configuration);
 }
 
 void
@@ -148,8 +139,8 @@ meta_wayland_shell_surface_managed (MetaWaylandShellSurface *shell_surface,
 }
 
 static void
-meta_wayland_shell_surface_surface_commit (MetaWaylandSurfaceRole  *surface_role,
-                                           MetaWaylandPendingState *pending)
+meta_wayland_shell_surface_surface_apply_state (MetaWaylandSurfaceRole  *surface_role,
+                                                MetaWaylandSurfaceState *pending)
 {
   MetaWaylandActorSurface *actor_surface =
     META_WAYLAND_ACTOR_SURFACE (surface_role);
@@ -162,7 +153,7 @@ meta_wayland_shell_surface_surface_commit (MetaWaylandSurfaceRole  *surface_role
 
   surface_role_class =
     META_WAYLAND_SURFACE_ROLE_CLASS (meta_wayland_shell_surface_parent_class);
-  surface_role_class->commit (surface_role, pending);
+  surface_role_class->apply_state (surface_role, pending);
 
   buffer = surface->buffer_ref.buffer;
   if (!buffer)
@@ -259,7 +250,8 @@ meta_wayland_shell_surface_class_init (MetaWaylandShellSurfaceClass *klass)
 
   object_class->finalize = meta_wayland_shell_surface_finalize;
 
-  surface_role_class->commit = meta_wayland_shell_surface_surface_commit;
+  surface_role_class->apply_state =
+    meta_wayland_shell_surface_surface_apply_state;
 
   actor_surface_class->get_geometry_scale =
     meta_wayland_shell_surface_get_geometry_scale;
