@@ -246,11 +246,7 @@ meta_x11_display_dispose (GObject *object)
       x11_display->gdk_display = NULL;
     }
 
-  if (x11_display->display_close_idle)
-    {
-      g_source_remove (x11_display->display_close_idle);
-      x11_display->display_close_idle = 0;
-    }
+  g_clear_handle_id (&x11_display->display_close_idle, g_source_remove);
 
   g_free (x11_display->name);
   x11_display->name = NULL;
@@ -1407,6 +1403,12 @@ meta_x11_display_get_screen_number (MetaX11Display *x11_display)
   return DefaultScreen (x11_display->xdisplay);
 }
 
+MetaDisplay *
+meta_x11_display_get_display (MetaX11Display *x11_display)
+{
+  return x11_display->display;
+}
+
 /**
  * meta_x11_display_get_xdisplay: (skip)
  * @x11_display: a #MetaX11Display
@@ -1931,8 +1933,8 @@ meta_x11_display_set_input_focus_xwindow (MetaX11Display *x11_display,
   if (meta_display_timestamp_too_old (x11_display->display, &timestamp))
     return;
 
-  serial = XNextRequest (x11_display->xdisplay);
   meta_x11_display_set_input_focus_internal (x11_display, window, timestamp);
+  serial = XNextRequest (x11_display->xdisplay);
   meta_x11_display_update_focus_window (x11_display, window, serial, TRUE);
   meta_display_update_focus_window (x11_display->display, NULL);
   meta_display_remove_autoraise_callback (x11_display->display);
