@@ -57,15 +57,12 @@ struct _MetaX11SelectionOutputStreamPrivate
 
   guint incr : 1;
   guint delete_pending : 1;
-  guint pipe_error : 1;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (MetaX11SelectionOutputStream,
                             meta_x11_selection_output_stream,
                             G_TYPE_OUTPUT_STREAM);
 
-static size_t get_element_size (int format);
-
 static void
 meta_x11_selection_output_stream_notify_selection (MetaX11SelectionOutputStream *stream)
 {
@@ -103,9 +100,6 @@ meta_x11_selection_output_stream_can_flush (MetaX11SelectionOutputStream *stream
 
   if (priv->delete_pending)
     return FALSE;
-  if (!g_output_stream_is_closing (G_OUTPUT_STREAM (stream)) &&
-      priv->data->len < get_element_size (priv->format))
-    return FALSE;
 
   return TRUE;
 }
@@ -119,7 +113,7 @@ get_max_request_size (MetaX11Display *display)
   if (size <= 0)
     size = XMaxRequestSize (display->xdisplay);
 
-  return (size - 100) * 4;
+  return size - 100;
 }
 
 static gboolean
@@ -129,12 +123,7 @@ meta_x11_selection_output_stream_needs_flush_unlocked (MetaX11SelectionOutputStr
     meta_x11_selection_output_stream_get_instance_private (stream);
 
   if (priv->data->len == 0)
-    {
-      if (priv->incr)
-        return g_output_stream_is_closing (G_OUTPUT_STREAM (stream));
-      else
-        return FALSE;
-    }
+    return FALSE;
 
   if (g_output_stream_is_closing (G_OUTPUT_STREAM (stream)))
     return TRUE;
@@ -182,34 +171,13 @@ get_element_size (int format)
     }
 }
 
-static gboolean
-meta_x11_selection_output_stream_check_pipe (MetaX11SelectionOutputStream  *stream,
-                                             GError                       **error)
-{
-  MetaX11SelectionOutputStreamPrivate *priv =
-    meta_x11_selection_output_stream_get_instance_private (stream);
-
-  if (priv->pipe_error)
-    {
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_BROKEN_PIPE,
-                   "Connection with client was broken");
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 static void
 meta_x11_selection_output_stream_perform_flush (MetaX11SelectionOutputStream *stream)
 {
   MetaX11SelectionOutputStreamPrivate *priv =
     meta_x11_selection_output_stream_get_instance_private (stream);
   Display *xdisplay;
-  size_t element_size, n_elements, max_size;
-  gboolean first_chunk = FALSE;
-  int error_code;
+  size_t element_size, n_elements;
 
   g_assert (!priv->delete_pending);
 
@@ -222,12 +190,8 @@ meta_x11_selection_output_stream_perform_flush (MetaX11SelectionOutputStream *st
 
   element_size = get_element_size (priv->format);
   n_elements = priv->data->len / element_size;
-  max_size = get_max_request_size (priv->x11_display);
 
-  if (!priv->incr)
-    first_chunk = TRUE;
-
-  if (!priv->incr && priv->data->len > max_size)
+  if (!g_output_stream_is_closing (G_OUTPUT_STREAM (stream)))
     {
       XWindowAttributes attrs;
 
@@ -243,22 +207,14 @@ meta_x11_selection_output_stream_perform_flush (MetaX11SelectionOutputStream *st
       XChangeProperty (xdisplay,
                        priv->xwindow,
                        priv->xproperty,
-                       XInternAtom (priv->x11_display->xdisplay, "INCR", False),
+                       XInternAtom (priv->x11_display->xdisplay, "INCR", True),
                        32,
                        PropModeReplace,
                        (guchar *) &(long) { n_elements },
                        1);
-      priv->delete_pending = TRUE;
     }
   else
     {
-      size_t copy_n_elements;
-
-      if (priv->incr && priv->data->len > 0)
-        priv->delete_pending = TRUE;
-
-      copy_n_elements = MIN (n_elements, max_size / element_size);
-
       XChangeProperty (xdisplay,
                        priv->xwindow,
                        priv->xproperty,
@@ -266,45 +222,30 @@ meta_x11_selection_output_stream_perform_flush (MetaX11SelectionOutputStream *st
                        priv->format,
                        PropModeReplace,
                        priv->data->data,
-                       copy_n_elements);
-      g_byte_array_remove_range (priv->data, 0, copy_n_elements * element_size);
+                       n_elements);
+      g_byte_array_remove_range (priv->data, 0, n_elements * element_size);
+      if (priv->data->len < element_size)
+        priv->flush_requested = FALSE;
     }
 
-  if (first_chunk)
-    meta_x11_selection_output_stream_notify_selection (stream);
+  meta_x11_selection_output_stream_notify_selection (stream);
 
+  priv->delete_pending = TRUE;
   g_cond_broadcast (&priv->cond);
   g_mutex_unlock (&priv->mutex);
 
-  error_code = meta_x11_error_trap_pop_with_return (priv->x11_display);
+  /* XXX: handle failure here and report EPIPE for future operations on the stream? */
+  if (meta_x11_error_trap_pop_with_return (priv->x11_display))
+    g_warning ("Failed to flush selection output stream");
 
-  if (error_code != Success)
-    {
-      char error_str[100];
-
-      priv->flush_requested = FALSE;
-      priv->delete_pending = FALSE;
-      priv->pipe_error = TRUE;
-
-      if (priv->pending_task)
-        {
-          XGetErrorText (xdisplay, error_code, error_str, sizeof (error_str));
-          g_task_return_new_error (priv->pending_task,
-                                   G_IO_ERROR,
-                                   G_IO_ERROR_BROKEN_PIPE,
-                                   "Failed to flush selection output stream: %s",
-                                   error_str);
-          g_clear_object (&priv->pending_task);
-        }
-    }
-  else if (priv->pending_task && priv->data->len == 0 && !priv->delete_pending)
+  if (priv->pending_task)
     {
       size_t result;
 
-      priv->flush_requested = FALSE;
       result = GPOINTER_TO_SIZE (g_task_get_task_data (priv->pending_task));
       g_task_return_int (priv->pending_task, result);
-      g_clear_object (&priv->pending_task);
+      g_object_unref (priv->pending_task);
+      priv->pending_task = NULL;
     }
 }
 
@@ -313,17 +254,12 @@ meta_x11_selection_output_stream_invoke_flush (gpointer data)
 {
   MetaX11SelectionOutputStream *stream =
     META_X11_SELECTION_OUTPUT_STREAM (data);
-  MetaX11SelectionOutputStreamPrivate *priv =
-    meta_x11_selection_output_stream_get_instance_private (stream);
 
   if (meta_x11_selection_output_stream_needs_flush (stream) &&
       meta_x11_selection_output_stream_can_flush (stream))
     meta_x11_selection_output_stream_perform_flush (stream);
 
-  if (priv->delete_pending || priv->data->len > 0)
-    return G_SOURCE_CONTINUE;
-  else
-    return G_SOURCE_REMOVE;
+  return G_SOURCE_REMOVE;
 }
 
 static gssize
@@ -338,9 +274,6 @@ meta_x11_selection_output_stream_write (GOutputStream  *output_stream,
   MetaX11SelectionOutputStreamPrivate *priv =
     meta_x11_selection_output_stream_get_instance_private (stream);
 
-  if (!meta_x11_selection_output_stream_check_pipe (stream, error))
-    return -1;
-
   g_mutex_lock (&priv->mutex);
   g_byte_array_append (priv->data, buffer, count);
   g_mutex_unlock (&priv->mutex);
@@ -368,19 +301,12 @@ meta_x11_selection_output_stream_write_async (GOutputStream       *output_stream
     META_X11_SELECTION_OUTPUT_STREAM (output_stream);
   MetaX11SelectionOutputStreamPrivate *priv =
     meta_x11_selection_output_stream_get_instance_private (stream);
-  GError *error = NULL;
   GTask *task;
 
   task = g_task_new (stream, cancellable, callback, user_data);
   g_task_set_source_tag (task, meta_x11_selection_output_stream_write_async);
   g_task_set_priority (task, io_priority);
 
-  if (!meta_x11_selection_output_stream_check_pipe (stream, &error))
-    {
-      g_task_return_error (task, error);
-      return;
-    }
-
   g_mutex_lock (&priv->mutex);
   g_byte_array_append (priv->data, buffer, count);
   g_mutex_unlock (&priv->mutex);
@@ -391,11 +317,16 @@ meta_x11_selection_output_stream_write_async (GOutputStream       *output_stream
       g_object_unref (task);
       return;
     }
+  else if (!meta_x11_selection_output_stream_can_flush (stream))
+    {
+      g_assert (priv->pending_task == NULL);
+      priv->pending_task = task;
+      g_task_set_task_data (task, GSIZE_TO_POINTER (count), NULL);
+      return;
+    }
   else
     {
-      if (meta_x11_selection_output_stream_can_flush (stream))
-        meta_x11_selection_output_stream_perform_flush (stream);
-
+      meta_x11_selection_output_stream_perform_flush (stream);
       g_task_return_int (task, count);
       g_object_unref (task);
       return;
@@ -423,7 +354,7 @@ meta_x11_selection_output_request_flush (MetaX11SelectionOutputStream *stream)
 
   g_mutex_lock (&priv->mutex);
 
-  if (priv->data->len > 0)
+  if (priv->data->len >= get_element_size (priv->format))
     priv->flush_requested = TRUE;
 
   needs_flush = meta_x11_selection_output_stream_needs_flush_unlocked (stream);
@@ -442,8 +373,6 @@ meta_x11_selection_output_stream_flush (GOutputStream  *output_stream,
   MetaX11SelectionOutputStreamPrivate *priv =
     meta_x11_selection_output_stream_get_instance_private (stream);
 
-  if (!meta_x11_selection_output_stream_check_pipe (stream, error))
-    return FALSE;
   if (!meta_x11_selection_output_request_flush (stream))
     return TRUE;
 
@@ -469,19 +398,12 @@ meta_x11_selection_output_stream_flush_async (GOutputStream       *output_stream
     META_X11_SELECTION_OUTPUT_STREAM (output_stream);
   MetaX11SelectionOutputStreamPrivate *priv =
     meta_x11_selection_output_stream_get_instance_private (stream);
-  GError *error = NULL;
   GTask *task;
 
   task = g_task_new (stream, cancellable, callback, user_data);
   g_task_set_source_tag (task, meta_x11_selection_output_stream_flush_async);
   g_task_set_priority (task, io_priority);
 
-  if (!meta_x11_selection_output_stream_check_pipe (stream, &error))
-    {
-      g_task_return_error (task, error);
-      return;
-    }
-
   if (!meta_x11_selection_output_stream_can_flush (stream))
     {
       if (meta_x11_selection_output_request_flush (stream))
@@ -498,9 +420,10 @@ meta_x11_selection_output_stream_flush_async (GOutputStream       *output_stream
         }
     }
 
-  g_assert (priv->pending_task == NULL);
-  priv->pending_task = task;
   meta_x11_selection_output_stream_perform_flush (stream);
+  g_task_return_boolean (task, TRUE);
+  g_object_unref (task);
+  return;
 }
 
 static gboolean
